@@ -1,5 +1,4 @@
 package POE::Component::DebugShell;
-# $Header: /cvsroot/sungo/POE-Component-DebugShell/lib/POE/Component/DebugShell.pm,v 1.18 2004/03/07 01:45:47 sungo Exp $
 
 use warnings;
 use strict;
@@ -12,8 +11,7 @@ use POE;
 use POE::Wheel::ReadLine;
 use POE::API::Peek;
 
-
-our $VERSION = (qw($Revision: 1.18 $))[1];
+our $VERSION = '1.412';
 our $RUNNING = 0;
 our %COMMANDS;
 our $SPAWN_TIME;
@@ -30,7 +28,6 @@ sub spawn { #{{{
     }
     # }}}
 
-    
     my $api = POE::API::Peek->new() or croak "Unable to create POE::API::Peek object";
 
 
@@ -59,13 +56,17 @@ sub spawn { #{{{
 
 
 sub _start { #{{{
-    $_[KERNEL]->alias_set(__PACKAGE__." controller");    
+    $_[KERNEL]->alias_set(__PACKAGE__." controller");
+
     $_[HEAP]->{rl} = POE::Wheel::ReadLine->new( InputEvent => 'term_input' );
     $_[HEAP]->{prompt} = 'debug> ';
-    
+
+    tie *STDOUT, "POE::Component::DebugShell::Output", 'stdout', \&_output;
+    tie *STDERR, "POE::Component::DebugShell::Output", 'stderr', \&_output;
+
     $_[HEAP]->{rl}->clear();
     _output("Welcome to POE Debug Shell v$VERSION");
-    
+
     $_[HEAP]->{rl}->get($_[HEAP]->{prompt});
 
 } #}}}
@@ -108,10 +109,10 @@ sub term_input { #{{{
         foreach my $cmd (sort keys %COMMANDS) {
             no warnings;
             my $short_help = $COMMANDS{$cmd}{short_help} || '[ No short help provided ]';
-            _output("\t* $cmd - $short_help"); 
+            _output("\t* $cmd - $short_help");
         }
         _output(' ');
-        
+
     } else  {
         my ($cmd, @args);
         if($input =~ /^(.+?)\s+(.*)$/) {
@@ -136,14 +137,14 @@ sub term_input { #{{{
     }
 
     $_[HEAP]->{rl}->get($_[HEAP]->{prompt});
-       
+
 } #}}}
 
 
 
 sub _output { #{{{
     my $msg = shift || ' ';
-    my $heap = $poe_kernel->get_active_session->get_heap;
+    my $heap = $poe_kernel->alias_resolve(__PACKAGE__." controller")->get_heap(); 
     $heap->{rl}->put($msg);
 } #}}}
 
@@ -166,7 +167,7 @@ sub _raw_commands { #{{{
         short_help => "Reload the shell to catch updates.",
         cmd => \&cmd_reload,
     },
-    
+
     show_sessions => {
         help => 'Show a list of all sessions in the system. The output format is in the form of loggable session ids.',
         short_help => 'Show a list of all sessions',
@@ -206,7 +207,7 @@ sub cmd_reload { #{{{
     eval q|
         no warnings qw(redefine);
         $SIG{__WARN__} = sub { };
-        
+
         foreach my $key (keys %INC) {
             if($key =~ m#POE/Component/DebugShell#) {
                 delete $INC{$key};
@@ -224,16 +225,15 @@ sub cmd_reload { #{{{
 sub cmd_show_sessions { #{{{
     my %args = @_;
     my $api = $args{api};
-    
-    my $ret;  
+
+    my $ret;
     $ret .= "Session List:\n";
     my @sessions = $api->session_list;
     foreach my $sess (@sessions) {
-        
         my $id = $sess->ID. " [ ".$api->session_id_loggable($sess)." ]";
         $ret .= "\t* $id\n";
     }
-    
+
     return $ret;
 } #}}}
 
@@ -241,9 +241,9 @@ sub cmd_list_aliases { #{{{
     my %args = @_;
     my $user_args = $args{args};
     my $api = $args{api};
-    
+
     my $ret;
-    
+
     if(my $id = shift @$user_args) {
         if(my $sess = $api->resolve_session_to_ref($id)) {
             my @aliases = $api->session_alias_list($sess);
@@ -273,7 +273,7 @@ sub cmd_session_stats { #{{{
     my $api = $args{api};
 
     my $ret;
-    
+
     if(my $id = shift @$user_args) {
         if(my $sess = $api->resolve_session_to_ref($id)) {
             my $to = $api->event_count_to($sess);
@@ -281,7 +281,7 @@ sub cmd_session_stats { #{{{
             $ret .= "Statistics for Session $id\n";
             $ret .= "\tEvents coming from: $from\n";
             $ret .= "\tEvents going to: $to\n";
-            
+
         } else {
             $ret .= "** Error: ID $id does not resolve to a session. Sorry.\n";
         }
@@ -290,7 +290,7 @@ sub cmd_session_stats { #{{{
     } else {
         $ret .= "** Error: Please provide a session id\n";
     }
-    
+
     return $ret;
 } #}}}
 
@@ -298,9 +298,9 @@ sub cmd_queue_dump { #{{{
     my %args = @_;
     my $api = $args{api};
     my $verbose;
-   
+
     my $ret;
-    
+
     if($args{args} && defined $args{args}) {
         if(ref $args{args} eq 'ARRAY') {
             if(@{$args{args}}[0] eq '-v') {
@@ -308,11 +308,11 @@ sub cmd_queue_dump { #{{{
             }
         }
     }
-    
+
     my @queue = $api->event_queue_dump();
-    
+
     $ret .= "Event Queue:\n";
-  
+
     foreach my $item (@queue) {
         $ret .= "\t* ID: ". $item->{ID}." - Index: ".$item->{index}."\n";
         $ret .= "\t\tPriority: ".$item->{priority}."\n";
@@ -347,6 +347,29 @@ sub cmd_status { #{{{
 
 # }}}
 
+1;
+
+package POE::Component::DebugShell::Output;
+
+use strict;
+#use warnings FATAL => "all";
+
+sub PRINT { 
+    my $self = shift;
+
+    my $txt = join('',@_);
+    $txt =~ s/\r?\n$//;
+    $self->{print}->($self->{type}."> $txt");
+}
+
+sub TIEHANDLE {
+    my $class = shift;
+    bless({
+        type => shift,
+        print => shift,
+    }, $class);
+}
+
 1;
 __END__
 
@@ -412,7 +435,7 @@ parameter.
         * ID: 704 - Index: 1
             Priority: 1078459012.42691
             Event: ping
-            
+
 Dump the contents of the event queue. Add a C<-v> parameter to get
 verbose output.
 
@@ -475,13 +498,11 @@ passed to it containing an C<api> parameter, which is a
 C<POE::API::Peek> object, and an C<args> parameter, which is an array
 reference of arguments (think C<@ARGV>).
 
+=back
+
 =head1 AUTHOR
 
-Matt Cashner (cpan@eekeek.org)
-
-=head1 DATE
-
-$Date: 2004/03/07 01:45:47 $
+Matt Cashner (sungo@pobox.com)
 
 =head1 LICENSE
 
@@ -511,4 +532,4 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
-
+# sungo // vim: ts=4 sw=4 expandtab
